@@ -431,6 +431,7 @@ def factorize_(
             # pd.cut with bins = IntervalIndex[datetime64] doesn't work...
             if groupvar.dtype.kind == "M":
                 expect = np.concatenate([expect.left.to_numpy(), [expect.right[-1].to_numpy()]])
+            # code is -1 for values outside the bounds of all intervals
             idx = pd.cut(groupvar.ravel(), bins=expect).codes.copy()
         else:
             if expect is not None and reindex:
@@ -455,9 +456,12 @@ def factorize_(
     grp_shape = tuple(len(grp) for grp in found_groups)
     ngroups = np.prod(grp_shape)
     if len(by) > 1:
-        group_idx = np.ravel_multi_index(factorized, grp_shape, mode="wrap").reshape(by[0].shape)
-        nan_by_mask = reduce(np.logical_or, [isnull(b) for b in by])
+        group_idx = np.ravel_multi_index(factorized, grp_shape, mode="wrap")
+        # NaNs; as well as values outside the bins are coded by -1
+        # Restore these after the raveling
+        nan_by_mask = reduce(np.logical_or, [(f == -1) for f in factorized])
         group_idx[nan_by_mask] = -1
+        group_idx = group_idx.reshape(by[0].shape)
     else:
         group_idx = factorized[0]
 
@@ -1310,11 +1314,12 @@ def _lazy_factorize_wrapper(*by, **kwargs):
     return group_idx
 
 
-def _factorize_multiple(by, expected_groups, by_is_dask):
+def _factorize_multiple(by, expected_groups, by_is_dask, reindex):
     kwargs = dict(
         expected_groups=expected_groups,
         axis=None,  # always None, we offset later if necessary.
         fastpath=True,
+        reindex=reindex,
     )
     if by_is_dask:
         import dask.array
@@ -1325,7 +1330,9 @@ def _factorize_multiple(by, expected_groups, by_is_dask):
             meta=np.array((), dtype=np.int64),
             **kwargs,
         )
-        found_groups = tuple(None if is_duck_dask_array(b) else pd.unique(b) for b in by)
+        found_groups = tuple(
+            None if is_duck_dask_array(b) else pd.unique(b.reshape(-1)) for b in by
+        )
         grp_shape = tuple(len(e) for e in expected_groups)
     else:
         group_idx, found_groups, grp_shape = factorize_(by, **kwargs)
@@ -1477,7 +1484,10 @@ def groupby_reduce(
     if len(by) == 1 and not isinstance(expected_groups, tuple):
         expected_groups = (np.asarray(expected_groups),)
     elif len(expected_groups) != len(by):
-        raise ValueError("len(expected_groups) != len(by)")
+        raise ValueError(
+            f"Must have same number of `expected_groups` (received {len(expected_groups)}) "
+            f" and variables to group by (received {len(by)})."
+        )
 
     # We convert to pd.Index since that lets us know if we are binning or not
     # (pd.IntervalIndex or not)
@@ -1489,7 +1499,7 @@ def groupby_reduce(
     )
     if factorize_early:
         by, final_groups, grp_shape = _factorize_multiple(
-            by, expected_groups, by_is_dask=by_is_dask
+            by, expected_groups, by_is_dask=by_is_dask, reindex=reindex
         )
         expected_groups = (pd.RangeIndex(np.prod(grp_shape)),)
 
